//----------------------------------------------------------------------------

//Common control class

//----------------------------------------------------------------------------

#include "control.hpp"
#include "bright.hpp"
#include "eeprom.hpp"
#include "eemap.hpp"
#include "presets.hpp"

//---------------------------- Resources Used: -------------------------------

//The TControl class performs common control of the measurement process.
//The Execute methods of the TProcess class for each of the channels are
//called In a loop. These methods do ADC reading and digital processing of
//readings. The calculated values of the levels are updated at a frequency
//of 375 Hz, then the ready flags UpdL and UpdR are set. When the data of the
//two channels is ready, they are read. Depending on the display mode DispMode,
//certain values are read. Values that are not displayed remain at zero.
//Read values are shown on the display. At the same time, the preset switching
//service works. The full set of parameters for the selected preset is read
//from EEPROM and loaded into all modules.

//------------------------------ Variables: ----------------------------------

TBright *Bright;
TEeprom *Eeprom;

//----------------------------------------------------------------------------
//---------------------------- TControl class: -------------------------------
//----------------------------------------------------------------------------

//----------------------------- Constructor: ---------------------------------

TControl::TControl(void)
{
  Eeprom = new TEeprom();
  Display = new TDisplay();
  Keyboard = new TKeyboard();
  ProcessL = new TProcess(CHANL);
  ProcessR = new TProcess(CHANR);
  Bright = new TBright();

  Jumpers = PRESETS;
  Preset = PRESETS;
#ifdef USE_KEYBOARD
  ChangePreset(Eeprom->ReadP());
#else
  PresetsService();
#endif

#ifdef USE_LEDS_MUX
  Pin_Mux0.Init(OUT_PP_2M, OUT_LO);
  Pin_Mux1.Init(OUT_PP_2M, OUT_LO);
  Pin_Mux2.Init(OUT_PP_2M, OUT_LO);
  Pin_MuxOut.Init(IN_PULL, PULL_DN);
  MuxN = 0;
  MuxData = 0;
#endif

  InitTest();

  UpdL = 0;
  UpdR = 0;
  Upd = 0;
}

//--------------------------- Execute control: -------------------------------

void TControl::Execute(void)
{
  //data ready check for channel L:
  if(ProcessL->Execute()) UpdL = 1;
  //data ready check for channel R:
  if(ProcessR->Execute()) UpdR = 1;
  //if both channels are ready, display:
  if(UpdL && UpdR)
  {
    ReadValues();    //read measured values
    DisplayValues(); //display values
    UpdL = 0;        //ready flags clear
    UpdR = 0;
    Upd = 1;         //set common ready flag
  }

  //services execution:
  if(TSysTimer::Tick) //every system tick
  {
    PresetsService(); //switch presets
#ifdef USE_LEDS_MUX
    MuxService();     //LEDs control using MUX
#endif
  }
}

//----------------------------------------------------------------------------
//----------------------- Services Implementation: ---------------------------
//---------------- (Services are called every system tick) -------------------
//----------------------------------------------------------------------------

//------------------------ Presets switch service: ---------------------------

void TControl::PresetsService(void)
{
#ifdef USE_KEYBOARD
  uint8_t k = Keyboard->Execute();
  if(k == KBD_MODE)
  {
    uint8_t p = Preset;
    if(DispMode & DP_ABSMAX)
    {
      Preset = PRESETS;
    }
    else
    {
      if(++p == PRESETS) p = 0;
    }
    ChangePreset(p);
    if(p != Eeprom->ReadP())
      Eeprom->WriteP(p);
  }
  else if(k == KBD_STAT)
  {
    SetDispMode(DispMode | DP_ABSMAX);
  }
  else if(k == KBD_STATH)
  {
    StatReset();
  }
#else
  uint8_t j = Keyboard->Jumpers();
  if(Jumpers != j)
  {
    Jumpers = j;
    ChangePreset(j);
  }
#endif
}

//----------------------- LEDs control using MUX service: --------------------

#ifdef USE_LEDS_MUX
void TControl::MuxService(void)
{
  if(Pin_MuxOut) MuxData |= 1 << MuxN; //read MUX
  if(++MuxN == MUX_INS) //next MUX address
  {
    MuxN = 0;
    LedsSet = 0;
    //MuxIn0 - High Speed, MuxIn1 - Low Speed
    if((MuxData & 0xC0) == 0x40)      LedsSet |= TR0; //9
    else if((MuxData & 0xC0) == 0x80) LedsSet |= TR2; //38
    else                              LedsSet |= TR1; //19
    //MuxIn2 - ?, MuxIn3 - ?
    if(MuxData & 0x20)                LedsSet |= TR3; //Rev 1
    if(MuxData & 0x10)                LedsSet |= TR4; //Rev Cycle
    //MuxIn4 - NR
    if(MuxData & 0x08)                LedsSet |= TR5; //NR
    //MuxIn5 - REC
    if(!(MuxData & 0x04))             LedsSet |= TR6; //REC
    //MuxIn6 - ?
    if(!(MuxData & 0x02))             LedsSet |= TR7; //SRC
    else                              LedsSet |= TR8; //TAPE
    //MuxIn7 - MUTE
    if(MuxData & 0x01)                LedsSet |= TR9; //MUTE
    MuxData = 0;
    Display->Leds(LedsSet);
    //Display->Update() is called in Execute() method.
  }
  Pin_Mux0 = MuxN & 1;
  Pin_Mux1 = MuxN & 2;
  Pin_Mux2 = MuxN & 4;
}
#endif

//----------------------------------------------------------------------------
//---------------------------- Auxiliary Methods: ----------------------------
//----------------------------------------------------------------------------

//------------------------- Initial bars lighting: ---------------------------

void TControl::InitTest(void)
{
  Display->BarL(SEGS);
  Display->BarR(SEGS);
  Display->Update();
  Display->Leds(TSALL);
  Display->Update();
  TSysTimer::Delay_ms(TEST_ON);
  Display->Leds(SCALE);
  Display->Update();

  for(uint8_t i = SEGS; i > 0; i--)
  {
    Display->BarL(i);
    Display->BarR(i);
    Display->Update();
    TSysTimer::Delay_ms(TEST_FALL / SEGS);
  }
  Display->Leds(LedsSet);
  Display->Update();
}

//------------------------- Read measured values: ----------------------------

#pragma inline = forced
void TControl::ReadValues(void)
{
  if(DispMode & DP_BAR)
  {
    BarL = ProcessL->GetBar();
    BarR = ProcessR->GetBar();
  }
  if(DispMode & DP_PEAK)
  {
    PeakL = ProcessL->GetPeak();
    PeakR = ProcessR->GetPeak();
  }
  if(DispMode & DP_BARMAX)
  {
    BarMaxL  = ProcessL->GetBarMax();
    BarMaxR  = ProcessR->GetBarMax();
  }
  if(DispMode & DP_PEAKMAX)
  {
    PeakMaxL = ProcessL->GetPeakMax();
    PeakMaxR = ProcessR->GetPeakMax();
  }
  if(DispMode & DP_ABSMAX)
  {
    BarL = ProcessL->GetBarAbsMax();
    BarR = ProcessR->GetBarAbsMax();
    PeakL = ProcessL->GetPeakAbsMax();
    PeakR = ProcessR->GetPeakAbsMax();
    BarMaxL = 0;
    BarMaxR = 0;
    PeakMaxL = 0;
    PeakMaxR = 0;
  }
}

//---------------------------- Display level: --------------------------------

#pragma inline = forced
void TControl::DisplayValues(void)
{
  PeakSegL    = SEGS;
  PeakSegR    = SEGS;
  PeakMaxSegL = SEGS;
  PeakMaxSegR = SEGS;
  BarMaxSegL  = SEGS;
  BarMaxSegR  = SEGS;
  //PeakSegL    = SEGS;
  //PeakSegR    = SEGS;
  for(uint8_t i = 0; i < SEGS; i++)
  {
    int16_t dBc = dBcTable[i];
    Display->SegL(i, BarL >= dBc);
    Display->SegR(i, BarR >= dBc);
    if(PeakL >= dBc) PeakSegL = i;
    if(PeakR >= dBc) PeakSegR = i;
    if(PeakMaxL >= dBc) PeakMaxSegL = i;
    if(PeakMaxR >= dBc) PeakMaxSegR = i;
    if(BarMaxL >= dBc) BarMaxSegL = i;
    if(BarMaxR >= dBc) BarMaxSegR = i;
  }
  Display->SegL(PeakSegL, 1);
  Display->SegR(PeakSegR, 1);
  Display->SegL(PeakMaxSegL, 1);
  Display->SegR(PeakMaxSegR, 1);
  Display->SegL(BarMaxSegL, 1);
  Display->SegR(BarMaxSegR, 1);
  Display->Update();
}

//------------------------ dB to code convertion: ----------------------------

uint16_t TControl::dB2Code(uint16_t dB)
{
  if(dB > 8000) return(0);
  uint32_t y = DB_001[dB & 0xF];
  y = y *  DB_016[(dB >> 4)  & 0xF] / 65536;
  y = y *  DB_256[(dB >> 8)  & 0xF] / 65536;
  y = y * DB_4096[(dB >> 12) & 0x1] / 65536;
  y = y * 10000 / 65536;
  return(y);
}

//----------------------------------------------------------------------------
//-------------------------- Interface Methods: ------------------------------
//----------------------------------------------------------------------------

//----------------------------- Load table: ----------------------------------

void TControl::SetTab(uint8_t n, int16_t v)
{
  if(n < SEGS) dBTable[n] = v;
}

//-------------------------- Codes table update: -----------------------------

void TControl::ApplyTab(void)
{
  int16_t dB_Max = dBTable[SEGS - 1];
  for(uint8_t i = 0; i < SEGS; i++)
  {
    dBcTable[i] = dB2Code(dB_Max - dBTable[i]);
  }
}

//------------------------------ Read table: ---------------------------------

int16_t TControl::GetTab(uint8_t n)
{
  return(dBTable[n]);
}

//---------------------------- Switch preset: --------------------------------

void TControl::ChangePreset(uint8_t p)
{
  if(p < PRESETS && p != Preset)
  {
    Preset = p;
    Eeprom->SelectPreset(Preset);
    EERead();
  }
}

//--------------------------- Switch display mode: ---------------------------

void TControl::SetDispMode(uint8_t m)
{
  DispMode = m;
  BarL     = 0; BarR     = 0;
  PeakL    = 0; PeakR    = 0;
  BarMaxL  = 0; BarMaxR  = 0;
  PeakMaxL = 0; PeakMaxR = 0;
}

//---------------------------- Reset statistics: -----------------------------

void TControl::StatReset(void)
{
  ProcessL->ResetAbsMax();
  ProcessR->ResetAbsMax();
}

//------------------------------ LEDs on/off: --------------------------------

void TControl::SetLeds(uint16_t n)
{
  LedsSet = n;
  Display->Leds(LedsSet);
  Display->Update();
}

//------------------------------ Read EEPROM: --------------------------------

void TControl::EERead(void)
{
  TProcess::EERead(Preset);
  Bright->EERead(Preset);
  for(uint8_t i = 0; i < SEGS; i++)
    dBTable[i] = Eeprom->Read(EE_DB_TAB + i, DBTABLE[i]);
  ApplyTab();
  SetDispMode(Eeprom->Read(EE_DMODE, DMD_P[Preset]));
#ifndef USE_LEDS_MUX
  SetLeds(Eeprom->Read(EE_LEDS, LEDS_P[Preset]));
#endif
}

//----------------------------- Write EEPROM: --------------------------------

bool TControl::EESave(void)
{
  Eeprom->Error = 0;
  TProcess::EESave();
  Bright->EESave();
  for(uint8_t i = 0; i < SEGS; i++)
    Eeprom->Write(EE_DB_TAB + i, dBTable[i]);
  Eeprom->Write(EE_DMODE, DispMode);
  Eeprom->Write(EE_LEDS, LedsSet);
  Eeprom->Validate();
  return(Eeprom->Error);
}

//---------------------------- Defaults load: --------------------------------

void TControl::LoadDefaults(void)
{
  Eeprom->Invalidate();
  EERead();
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
